/***************************************************************************//**
 *   @file   ADXL362 sketch.
 *   @brief  Demo project for ADXL362 using MPIDE Serial Monitor.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code
 must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 730
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include <ADXL362.h>
#include <DSPI.h>

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/
/* List of available commands */
const char* commandsList[] = {"help?",
                              "id?",
                              "measure=",
                              "temp?",
                              "reset=",
                              "acceleration?",
                              "activity?"};
const char* commandsDescription[] = {
    "  -  Displays all available commands.",
    "  -  Device details.",
    "  -  Start/stop the measure process of the device. Accepted values: 0 - 1.",
    "  -  Read the temperature.",
    "  -  Reset the device.",
    "  -  Displays the accelerations on XYZ axis.",
    "  -  Displays the activity status of the device. It runs for 5 motion detections."};
    
/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
ADXL362 myADXL362;

/* Variables used for console operations */
char commandsNumber = (sizeof(commandsList) / sizeof(const char*));
char receivedCommand[20];
char invalidCommand = 0;
char commandType = 0;
char command = 0;
char displayCommand = 0;
double        commandParam = 0;
/* Variables holding information about the device */
double          xAxisAcc = 0;
double          yAxisAcc = 0;
double          zAxisAcc = 0;
unsigned char  regVal = 0;
double          degCelsius = 0;
/* Temporary variables */
char  tempString[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
unsigned long  samplesNumber  = 0;
unsigned char  detections     = 0;
long           intCommand     = 0;

/***************************************************************************//**
 * @brief Reads one command from UART.
 *
 * @param command - Read command.
 *
 * @return None.
*******************************************************************************/
void CONSOLE_GetCommand(char* command)

{
    unsigned char receivedChar = 0;
    
    unsigned char charNumber = 0;
    
    while(receivedChar != 0x0D)
    
    {
      if (Serial.available() > 0)
      {
        
        receivedChar = Serial.read();
        command[charNumber++] = receivedChar; 
      }
    }
    command[charNumber] = 0;
}

/***************************************************************************//** 
 * @param receivedCommand - Received command.
 * @param expectedCommand - Expected command.
 * @param commandParameter - Command parameter.
 *
 * @return commandType - Type of the command.
 *                       Example: 0 - Commands don't match.
 *                                1 - Write command.
 *                                2 - Read command.
*******************************************************************************/
unsigned char CONSOLE_CheckCommands(char* receivedCommand,
                                    const char* expectedCommand,
                                    double* commandParameter)
{
    unsigned char commandType = 1;
    unsigned char charIndex = 0;

    unsigned char parameterString[5] = {0, 0, 0, 0, 0};
    unsigned char parameterIndex = 0;

    while((expectedCommand[charIndex] != '?') &&
          (expectedCommand[charIndex] != '=') &&
          
          (commandType != 0))
    {
        if(expectedCommand[charIndex] != receivedCommand[charIndex])
        {
            commandType = 0;
        }
        charIndex++;
    }
    if(commandType != 0)
    {
        if(expectedCommand[charIndex] == '=')
        {
            if(receivedCommand[charIndex] == '=')
            {
                charIndex++;
     
                while(receivedCommand[charIndex] != 0x0D)
                {
                    parameterString[parameterIndex] = receivedCommand[charIndex];
                    charIndex++;
                    parameterIndex++;
                    
                }
                *commandParameter = atof((const char*)parameterString);
            }
            else
            {
                commandType = 0;
            }
        }
        if(expectedCommand[charIndex] == '?')
        {
            if(receivedCommand[charIndex] == '?')
            {
                    commandType = 2;
            }
            else
            {
                    commandType = 0;
            }
        }
    }

    return commandType;
}

/***************************************************************************//**
 * @brief Setup function.
 *
 * @return none.
*******************************************************************************/ 
void setup()
{
  Serial.begin(9600);
  if(myADXL362.Init())
    {
        Serial.println("ADXL362 OK");
    }
    else
    {
        Serial.println("ADXL362 Error");
    }
  myADXL362.SetRange(ADXL362_RANGE_4G);  
}

/***************************************************************************//**
 * @brief Loop function.
 *
 * @return none.
*******************************************************************************/
void loop()
{
  /* Wait a command from the user. */
  CONSOLE_GetCommand(receivedCommand);
  invalidCommand = 0;
  /* Check if the received command identifies as one of the command list.  */
  for(command = 0; command < commandsNumber; command++)
  {
      commandType = CONSOLE_CheckCommands(receivedCommand,
                                          commandsList[command],
                                          (double*)&commandParam);
      /* Count the commands that are invalid. */
      if(commandType == 0)
      {
          invalidCommand++;
      }
      else
      {
        switch(command)
        {
          case 0:                                        // "help?" command
              Serial.println("Available commands:");
              for(displayCommand = 0; displayCommand < commandsNumber;
                  displayCommand++)
              {
                  Serial.print(commandsList[displayCommand]);
                  Serial.println(commandsDescription[displayCommand]);
              }
              break;
          case 1:                                        // "id?" command
              intCommand = (long)commandParam;
              Serial.print("Analog Devices device ID: ");
              myADXL362.GetRegisterValue(&regVal, ADXL362_REG_DEVID_AD, 1);
              itoa(regVal, tempString, 16);
              Serial.println(tempString);
              Serial.print("Analog Devices MEMS ID: ");
              myADXL362.GetRegisterValue(&regVal, ADXL362_REG_DEVID_MST, 1);
              itoa(regVal, tempString, 16);
              Serial.println(tempString);
              Serial.print("Analog Devices PART ID: ");
              myADXL362.GetRegisterValue(&regVal, ADXL362_REG_PARTID, 1);
              itoa(regVal, tempString, 8);
              Serial.println(tempString);
              Serial.print("Analog Devices REVISION ID: ");
              myADXL362.GetRegisterValue(&regVal, ADXL362_REG_REVID, 1);
              itoa(regVal, tempString, 10);
              Serial.println(tempString);
              break;
          case 2:                                        // "measure=" command
              /* Turn on/off the measurement process of the device. */
              intCommand = (long)(commandParam);
              if((intCommand == 1) || (intCommand == 0))
              {
                  myADXL362.SetPowerMode(intCommand);
              }
              else
              {
                  intCommand = 0;
              }
              /* Send feedback to user. */
              Serial.print(commandsList[command]);
              itoa(intCommand, tempString, 10);
              Serial.println(tempString);
              break;
          case 3:                                        // "temperature?" command
              /* Read the temperature of the device. */
              degCelsius = myADXL362.ReadTemperature();
              sprintf(tempString, "%.2f", degCelsius);
              Serial.print(tempString);
              Serial.println(" deg. Celsius");
              break;
          case 4:                                        // "reset=" command
              /* Resets the device(software reset). */
              intCommand = (long)commandParam;
              if(intCommand == 1)
              {
                  myADXL362.SoftwareReset();
              }
              else
              {
                  intCommand = 0;
              }
              /* Send feedback to user */
              Serial.print(commandsList[command]);
              itoa(intCommand, tempString, 10);
              Serial.println(tempString);
              break;
          case 5:                                        // "acceleration?" command
              /* Read and displays 30 samples from the accelerometer. */
              samplesNumber = 30;
              while(samplesNumber)
              {
                  /* Read the requested value from the device. */
                  myADXL362.ReadAccel(&xAxisAcc, &yAxisAcc, &zAxisAcc);
                  /* Send the requested value to user. */
                  Serial.print("x axis: ");
                  sprintf(tempString, "%.3f", xAxisAcc);
                  Serial.print(tempString);
                  Serial.print(" [g]  ");
                  Serial.print("y axis: ");
                  sprintf(tempString, "%.3f", yAxisAcc);
                  Serial.print(tempString);
                  Serial.print(" [g]  ");
                  Serial.print("z axis: ");
                  sprintf(tempString, "%.3f", zAxisAcc);
                  Serial.print(tempString);
                  Serial.println(" [g]");
                  delay(100);
                  samplesNumber--;
              }
              break;
          case 6:                                        // "activity?" command
              /* Setup the activity and inactivity detection. */
              Serial.println("Activity polling enabled.");
              myADXL362.SetPowerMode(0);
              myADXL362.SetOutputRate(ADXL362_ODR_100_HZ);
              myADXL362.SetRegisterValue(
                            ADXL362_ACT_INACT_CTL_LINKLOOP(ADXL362_MODE_LINK),
                            ADXL362_REG_ACT_INACT_CTL,
                            1);
              myADXL362.SetupActivityDetection(1, 60, 4);
              myADXL362.SetupInactivityDetection(1, 700, 250);
              myADXL362.SetPowerMode(1);
              /* Clear ACT and INACT bits by reading the Status Register. */
              myADXL362.GetRegisterValue(&regVal,
                                         ADXL362_REG_STATUS,
                                         1);
              /* Exit polling after 5 detections. */
              detections = 5;
              while(detections)
              {
                  do // Wait for the detection of an activity or inactivity.
                  {
                      myADXL362.GetRegisterValue(&regVal,
                                                 ADXL362_REG_STATUS,
                                                 1);
                  }while(!(regVal & ADXL362_STATUS_ACT) &&
                         !(regVal & ADXL362_STATUS_INACT));
                  detections--;
                  if(regVal & ADXL362_STATUS_ACT)
                  {
                      Serial.println("Activity");
                  }
                  if(regVal & ADXL362_STATUS_INACT)
                  {
                      Serial.println("Inactivity");
                  }
              }
              Serial.println("Finished activity polling.");
              break;
         default:
              break;   
        }// End switch 
    }// End else
  }// End for
  if(invalidCommand == commandsNumber)
  {
      /* Send feedback to user */
       Serial.println("Invalid command");
  }  
}
